@@ -1,5 +1,14 @@
+#!/usr/bin/env python3
+# -*- coding: utf-8 -*-
+
+"""
+Written by Lucas Sinclair.
+MIT Licensed.
+Contact at www.sinclair.bio
+"""
+
 # Special variables #
-__version__ = '1.3.8'
+__version__ = '1.5.9'
 
 # Built in modules #
 import os
