@@ -1,5 +1,14 @@
+#!/usr/bin/env python3
+# -*- coding: utf-8 -*-
+
+"""
+Written by Lucas Sinclair.
+MIT Licensed.
+Contact at www.sinclair.bio
+"""
+
 # Built-in modules #
-import os, tempfile, subprocess, shutil, codecs, gzip, zipfile, hashlib
+import os, tempfile, subprocess, shutil, gzip, zipfile, hashlib
 
 # Internal modules #
 import autopaths
@@ -9,11 +18,6 @@ from autopaths.common import pad_extra_whitespace
 if os.name == "posix": sep = "/"
 if os.name == "nt":    sep = "\\"
 
-# Python 2 compatibility #
-import six
-if six.PY2:
-    from io import open
-
 ################################################################################
 class FilePath(autopaths.base_path.BasePath):
     """
@@ -33,7 +37,6 @@ class FilePath(autopaths.base_path.BasePath):
     """
 
     def __bool__(self): return self.path is not None and self.count_bytes != 0
-    __nonzero__ = __bool__
 
     def __iter__(self):
         with open(self.path, 'r', encoding='utf-8') as handle:
@@ -52,9 +55,9 @@ class FilePath(autopaths.base_path.BasePath):
 
     def __enter__(self):
         """Called when entering the 'with' statement (context manager)."""
-        return self
+        return self.open()
 
-    def __exit__(self, errtype, value, traceback):
+    def __exit__(self, err_type, value, traceback):
         """
         Called when exiting the 'with' statement.
         This enables us to close the file or database properly, even when
@@ -71,14 +74,21 @@ class FilePath(autopaths.base_path.BasePath):
 
     @property
     def prefix_path(self):
-        """The full path without the (last) extension and trailing period."""
+        """The full path without the last extension and trailing period."""
         return str(os.path.splitext(self.path)[0])
 
     @property
     def prefix(self):
-        """Just the filename without the (last) extension and trailing period."""
+        """Just the filename without the last extension and trailing period."""
         return str(os.path.basename(self.prefix_path))
 
+    @property
+    def extension(self):
+        """Just the last extension without the trailing period."""
+        if '.' not in self.filename:
+            raise Exception("The file '%s' has no extension." % self.path)
+        return self.filename.split('.')[-1]
+
     @property
     def name(self):
         """Shortcut for self.filename."""
@@ -89,11 +99,6 @@ class FilePath(autopaths.base_path.BasePath):
         """Just the filename with the extension."""
         return str(os.path.basename(self.path))
 
-    @property
-    def extension(self):
-        """The extension with the leading period."""
-        return os.path.splitext(self.path)[1]
-
     @property
     def directory(self):
         """The directory containing this file."""
@@ -115,10 +120,12 @@ class FilePath(autopaths.base_path.BasePath):
         """We are going to default to the number of lines."""
         # Third party modules #
         if os.name == "posix": import sh
-        if os.name == "nt":    import pbs
+        if os.name == "nt":    import pbs3
         # Count lines #
-        if os.name == "posix": return int(sh.wc('-l', self.path).split()[0])
-        if os.name == "nt":    return int(pbs.Command("find")('/c', '/v', '""', self.path))
+        if os.name == "posix":
+            return int(sh.wc('-l', self.path).split()[0])
+        if os.name == "nt":
+            return int(pbs3.Command("find")('/c', '/v', '""', self.path))
 
     @property
     def size(self):
@@ -135,25 +142,6 @@ class FilePath(autopaths.base_path.BasePath):
         """The contents as a unicode string."""
         with open(self.path, encoding='utf8') as handle:return handle.read()
 
-    @property
-    def absolute_path(self):
-        """The absolute path starting with a `/`."""
-        return FilePath(os.path.abspath(self.path))
-
-    @property
-    def physical_path(self):
-        """The physical path like in `pwd -P`."""
-        return FilePath(os.path.realpath(self.path))
-
-    @property
-    def relative_path(self):
-        """The relative path when compared with current directory."""
-        return FilePath(os.path.relpath(self.physical_path))
-
-    def rel_path_from(self, path):
-        """The relative path when compared to the given path."""
-        return FilePath(os.path.relpath(self.path, path))
-
     @property
     def md5(self):
         """Compute the md5 of a file. Pretty fast."""
@@ -214,13 +202,15 @@ class FilePath(autopaths.base_path.BasePath):
         if encoding is None:
             with open(self.path, mode) as handle: handle.write(content)
         else:
-            with open(self.path, mode, encoding=encoding) as handle: handle.write(content)
+            with open(self.path, mode, encoding=encoding) as handle:
+                handle.write(content)
 
     def writelines(self, content, encoding=None, mode='w'):
         if encoding is None:
             with open(self.path, mode) as handle: handle.writelines(content)
         else:
-            with open(self.path, mode, encoding=encoding) as handle: handle.writelines(content)
+            with open(self.path, mode, encoding=encoding) as handle:
+                handle.writelines(content)
 
     def remove(self):
         if not self.exists: return False
@@ -246,12 +236,16 @@ class FilePath(autopaths.base_path.BasePath):
         return self.prefix_path + "." + string + self.extension
 
     def make_directory(self):
-        """Create the directory the file is supposed to be in if it does not exist."""
+        """
+        Create the directory the file is supposed to be in if it does not
+        exist.
+        """
         if not self.directory.exists: self.directory.create()
 
     def must_exist(self):
         """Raise an exception if the path doesn't exist."""
-        if not self.exists: raise Exception("The file path '%s' does not exist." % self.path)
+        if not self.exists:
+            raise Exception("The file path '%s' does not exist." % self.path)
 
     def head(self, num_lines=20):
         """Yield the first few lines."""
@@ -298,14 +292,20 @@ class FilePath(autopaths.base_path.BasePath):
 
     def move_to(self, path, overwrite=False):
         """Move the file to a new location."""
+        # Parse the path #
+        path = autopaths.Path(path)
         # Special directory case, keep the same name (put it inside) #
         if path.endswith(sep): path = path + self.filename
+        # Check that the directory exists #
+        path.directory.create_if_not_exists()
         # Normal case #
         if os.path.exists(path) and overwrite: os.remove(path)
         assert not os.path.exists(path)
         shutil.move(self.path, path)
         # Update the internal link #
         self.path = path
+        # Return #
+        return path
 
     def rename(self, new_name):
         """Rename the file but leave it in the same directory."""
@@ -319,29 +319,37 @@ class FilePath(autopaths.base_path.BasePath):
     #------------------------------ Compression ------------------------------#
     def gzip_to(self, path=None):
         """Make a gzipped version of the file at a given path."""
+        # Case where path is not specified #
         if path is None: path = self.path + ".gz"
+        # Do it #
         with open(self.path, 'rb') as orig_file:
             with gzip.open(path, 'wb') as new_file:
                 new_file.writelines(orig_file)
+        # Return #
         return FilePath(path)
 
-    def ungzip_to(self, path=None, mode='w'):
+    def ungzip_to(self, path=None, mode='wb'):
         """Make an unzipped version of the file at a given path."""
-        if path is None: path = self.path[:3]
+        # Case where path is not specified #
+        if path is None: path = self.path[:-3]
+        # Do it #
         with gzip.open(self, 'rb') as orig_file:
             with open(path, mode) as new_file:
                 new_file.writelines(orig_file)
+        # Return #
         return FilePath(path)
 
     def zip_to(self, path=None):
         """Make a zipped version of the file at a given path."""
         pass
 
-    def unzip_to(self, destination=None, inplace=False, single=True):
+    def unzip_to(self, path=None, inplace=False, single=True):
         """
         Unzip a standard zip file. Can specify the destination of the
         uncompressed file, or just set inplace=True to delete the original.
         """
+        # Parse the path #
+        path = autopaths.Path(path)
         # Check #
         assert zipfile.is_zipfile(self.path)
         # Load #
@@ -353,31 +361,54 @@ class FilePath(autopaths.base_path.BasePath):
             tmpdir = tempfile.mkdtemp() + sep
             z.extract(member, tmpdir)
             z.close()
-            if inplace: shutil.move(tmpdir + member.filename, self.directory + member.filename)
-            else:       shutil.move(tmpdir + member.filename, destination)
-        # Multifile - no security, dangerous - Will use CWD if dest is None!! #
-        # If a file starts with an absolute path, will overwrite your files anywhere #
+            if inplace: shutil.move(tmpdir + member.filename,
+                                    self.directory + member.filename)
+            else:       shutil.move(tmpdir + member.filename,
+                                    path)
+        # Multifile - no security, dangerous - Will use CWD if dest is None!!
+        # If a file starts with an absolute path, will overwrite your files
+        # anywhere
         if not single:
-            z.extractall(destination)
+            z.extractall(path)
+            return autopaths.dir_path.DirectoryPath(path)
+        # Return #
+        return FilePath(path)
 
     def targz_to(self, path=None):
         """Make a targzipped version of the file at a given path."""
         pass
 
-    def untargz_to(self, destination=None):
+    def untargz_to(self, path=None):
         """Make an untargzipped version of the file at a given path."""
+        # Case where path is not specified #
+        if path is None:
+            if   self.path.endswith('.tgz'):    path = self.path[:-4]
+            elif self.path.endswith('.tar.gz'): path = self.path[:-7]
+            else: path = self.path + '.untargz'
+        # Do it #
         import tarfile
-        archive = tarfile.open(self.path, 'r:gz')
-        archive.extractall(destination)
+        with tarfile.open(self.path, 'r:gz') as archive:
+            archive.extractall(path)
+        # Return #
+        return autopaths.dir_path.DirectoryPath(path + '/')
 
     def tar_to(self, path=None):
         """Make a tared version of the file at a given path."""
         pass
 
-    def untar_to(self, destination=None):
+    def untar_to(self, path=None):
         """Make an untared version of the file at a given path."""
+        # Case where path is not specified #
+        if path is None:
+            if   self.path.endswith('.tgz'):    path = self.path[:-4]
+            elif self.path.endswith('.tar.gz'): path = self.path[:-7]
+            else: path = self.path + '.untargz'
+        # Do it #
         import tarfile
-        with tarfile.open(self.path) as archive: archive.extractall(destination)
+        with tarfile.open(self.path) as archive:
+            archive.extractall(path)
+        # Return #
+        return autopaths.dir_path.DirectoryPath(path + '/')
 
     #-------------------------------- Modify ---------------------------------#
     def append(self, data):
@@ -435,7 +466,7 @@ class FilePath(autopaths.base_path.BasePath):
         self.remove()
         result_file.move_to(self)
 
-    def replace_line(self, line_to_remove, line_to_insert):
+    def replace_line(self, line_to_remove, line_to_insert, safe=False):
         """
         Search the file for a given line, and if found,
         replace it with another line.
@@ -447,9 +478,15 @@ class FilePath(autopaths.base_path.BasePath):
         result_file = autopaths.tmp_path.new_temp_file()
         # Generate the lines #
         def new_lines():
+            found = False
             for line in self:
-                if line.strip() == line_to_remove: yield line_to_insert + '\n'
-                else:                              yield line
+                if line.rstrip() == line_to_remove.rstrip():
+                    yield line_to_insert.rstrip() + '\n'
+                    found = True
+                else: yield line
+            if found is False and safe is False:
+                msg = "The line to replace ('%s') was not found in '%s'"
+                raise Exception(msg % (line_to_remove, self.path))
         # Open input/output files, note: output file's permissions lost #
         result_file.writelines(new_lines())
         # Switch the files around #
@@ -479,6 +516,6 @@ class FilePath(autopaths.base_path.BasePath):
             import sh
             return sh.sed('-i', 's/%s/%s/' % (before, after), self.path)
         if os.name == "nt":
-            import pbs
+            import pbs3
             sed_cmd = 'sed -i "s/%s/%s/" %s' % (before, after, self.path)
-            return pbs.bash('-c', "'" + sed_cmd + "'" )
+            return pbs3.bash('-c', "'" + sed_cmd + "'" )
